@@ -1,11 +1,10 @@
 package App::ElasticSearch::Utilities::Aggregations;
-# ABSTRACT: Code to simplify creating and working with Elasticsearh aggregations
+# ABSTRACT: Code to simplify creating and working with Elasticsearch aggregations
 
 use strict;
 use warnings;
 
 use Storable qw(dclone);
-
 use Sub::Exporter -setup => {
     exports => [ qw(
         expand_aggregate_string
@@ -21,57 +20,92 @@ use Sub::Exporter -setup => {
     },
 };
 
-my %Aggregations = (
-    avg => { single_stat => 1 },
-    cardinality => { single_stat => 1 },
-    date_histogram => {
-        params => sub { { calendar_interval => $_[0] || '1h' } },
+my %Aggregations;
+
+
+$Aggregations{terms} = {
+    params    => sub { $_[0] && $_[0] =~ /^\d+$/ ? { size => $_[0] } : {} },
+    type      => 'bucket',
+    composite => 1,
+};
+
+
+$Aggregations{significant_terms} = {
+    params => sub { $_[0] =~ /^\d+$/ ? { size => $_[0] } : {} },
+    type   => 'bucket',
+};
+
+
+$Aggregations{rare_terms} = {
+    params => sub { $_[0] =~ /^\d+$/ ? { max_doc_count => $_[0] } : {} },
+    type   => 'bucket',
+};
+
+
+$Aggregations{histogram} = {
+    params => sub {
+        return unless $_[0] > 0;
+        return { interval => $_[0] };
     },
-    extend_stats => {},
-    geo_centroid => {},
-    geohash_grid => {
-        params => sub { $_[0] =~ /^\d+$/ ? { precision => $_[0] } : {} },
+    type      => 'bucket',
+    composite => 1,
+};
+
+
+$Aggregations{date_histogram} = {
+    params    => sub { { calendar_interval => $_[0] || '1h' } },
+    type      => 'bucket',
+    composite => 1,
+};
+
+
+$Aggregations{geohash_grid} = {
+    params    => sub { $_[0] =~ /^\d+$/ ? { precision => $_[0] } : {} },
+    type      => 'bucket',
+    composite => 1,
+};
+
+
+$Aggregations{missing} = { type => 'bucket' };
+
+
+$Aggregations{avg} = { single_stat => 1, type => 'metric' };
+$Aggregations{max} = { single_stat => 1, type => 'metric' };
+$Aggregations{min} = { single_stat => 1, type => 'metric' };
+$Aggregations{sum} = { single_stat => 1, type => 'metric' };
+
+
+$Aggregations{cardinality} = { single_stat => 1, type => 'metric' };
+
+
+$Aggregations{stats} = { type => 'metric' };
+
+
+$Aggregations{extended_stats} = { type => 'metric' };
+
+
+$Aggregations{percentiles} = {
+    params => sub {
+        my @pcts = $_[0] ? split /,/, $_[0] : qw(25 50 75 90);
+        return { percents => \@pcts };
     },
-    histogram => {
-        params => sub {
-            return unless $_[0] > 0;
-            return { interval => $_[0] };
-        },
-    },
-    max => { single_stat => 1 },
-    min => { single_stat => 1 },
-    missing => { single_stat => 1 },
-    percentiles => {
-        params => sub {
-            my @pcts = $_[0] ? split /,/, $_[0] : qw(25 50 75 90);
-            return { percents => \@pcts };
-        },
-    },
-    rare_terms => {
-        params => sub { $_[0] =~ /^\d+$/ ? { max_doc_count => $_[0] } : {} },
-    },
-    terms => {
-        params => sub { $_[0] =~ /^\d+$/ ? { size => $_[0] } : {} },
-    },
-    significant_terms => {
-        params => sub { $_[0] =~ /^\d+$/ ? { size => $_[0] } : {} },
-    },
-    stats => {},
-    sum => { single_stat => 1 },
-    weighted_avg => {},
-);
+};
+
+
+$Aggregations{geo_centroid} = { type => 'metric' };
+
+
 
 
 sub is_single_stat {
     my ($agg) = @_;
     return unless $agg;
     return unless exists $Aggregations{$agg};
-    return unless exists $Aggregations{$agg}{single_stat};
-    return $Aggregations{$agg}{single_stat};
+    return unless exists $Aggregations{$agg}->{single_stat};
+    return $Aggregations{$agg}->{single_stat};
 }
 
 
-
 sub expand_aggregate_string {
     my ($token) = @_;
 
@@ -108,7 +142,7 @@ sub expand_aggregate_string {
             # Process parameters
             $params = $Aggregations{$agg}->{params}->($paramStr);
         }
-        $alias ||= join "_", $agg eq 'terms' ? ($field) : ($agg, $field);
+        $alias ||= join ".", $agg eq 'terms' ? ($field) : ($agg, $field);
         $aggs{$alias} = { $agg => { field => $field, %{ $params } } };
     }
     return \%aggs;
@@ -211,11 +245,11 @@ __END__
 
 =head1 NAME
 
-App::ElasticSearch::Utilities::Aggregations - Code to simplify creating and working with Elasticsearh aggregations
+App::ElasticSearch::Utilities::Aggregations - Code to simplify creating and working with Elasticsearch aggregations
 
 =head1 VERSION
 
-version 8.0
+version 8.1
 
 =head1 FUNCTIONS
 
@@ -296,6 +330,260 @@ It returns an array reference, containing arrray references.  The individual
 rows of the array are ordered in a depth first fashion.  The array does include
 a key for every value, so the array can be cast to a hash safely.
 
+=head1 Aggregations
+
+List of supported aggregations.  Other aggregation may work, but these have defined behavior.
+
+=head2 Bucket Aggregations
+
+These aggregations will support sub aggregations.
+
+=over 2
+
+=item B<terms>
+
+The default aggregation if none is specified.
+
+    field_name
+    terms:field_name
+
+Results in
+
+    {
+        "field_name": {
+            "terms": {
+                "field": "field_name"
+            }
+        }
+    }
+
+Supports a positional parameter: size
+
+    field_name:20
+    terms:field_name:20
+
+Results in
+
+    {
+        "field_name": {
+            "terms": {
+                "field": "field_name",
+                "size": 20
+            }
+        }
+    }
+
+=item B<significant_terms>
+
+Same as C<terms>.
+
+    significant_terms:field_name:10
+
+Results in:
+
+    {
+        "rare_terms.field_name": {
+            "terms": {
+                "field": "field_name",
+                "size": 10
+            }
+        }
+    }
+
+=item B<rare_terms>
+
+Same as C<terms> but the positional parameter is the C<max_doc_count>.
+
+    rare_terms:field_name:10
+
+Results in:
+
+    {
+        "rare_terms.field_name": {
+            "terms": {
+                "field": "field_name",
+                "max_doc_count": 10
+            }
+        }
+    }
+
+=item B<histogram>
+
+Creates a histogram for numeric fields.  Positional parameter is the interval.
+
+    histogram:field_name:10
+
+Results in:
+
+    {
+        "histogram.field_name": {
+            "histogram": {
+                "field": "field_name",
+                "interval": 10
+            }
+        }
+    }
+
+=item B<date_histogram>
+
+Creates a histogram for date fields.  Positional parameter is the calendar_interval.
+
+    date_histogram:field_name:1h
+
+Results in:
+
+    {
+        "histogram.field_name": {
+            "histogram": {
+                "field": "field_name",
+                "calendar_interval": "1h"
+            }
+        }
+    }
+
+=item B<geohash_grid>
+
+Creates a geohash grid bucket aggregation.  Positional parameter is the precision.
+
+    geohash_grid:field_name:6
+
+Results in:
+
+    {
+        "geohash_grid.field_name": {
+            "geohash_grid": {
+                "field": "field_name",
+                "precision": 6
+            }
+        }
+    }
+
+=item B<missing>
+
+Creates a bucket for documents missing the field.  No positional parameters.
+
+    missing:field_name
+
+Results in:
+
+    {
+        "missing.field_name": {
+            "missing": {
+                "field": "field_name"
+            }
+        }
+    }
+
+=back
+
+=head2 Metric Aggregations
+
+Aggregations that generate metrics from enclosing buckets.
+
+=over 2
+
+=item B<avg>, B<max>, B<min>, B<sum>
+
+Single stat metric aggregations to generate the various single statistics over the enclosing bucket.
+
+    sum:field_name
+
+Results in
+
+    {
+        "sum.field_names": {
+            "sum": {
+                "field": "field_name"
+            }
+        }
+    }
+
+=item B<cardinality>
+
+Computes the unique count of terms in a field.
+
+    cardinality:field_name
+
+Results in
+
+    {
+        "cardinality.field_names": {
+            "cardinality": {
+                "field": "field_name"
+            }
+        }
+    }
+
+=item B<stats>
+
+Runs the stats aggregation that returns min, max, avg, sum, and count.
+
+    stats:field_name
+
+Results in
+
+    {
+        "stats.field_names": {
+            "stats": {
+                "field": "field_name"
+            }
+        }
+    }
+
+=item B<extended_stats>
+
+Runs the stats aggregation that returns the same data as the C<sum> aggregation
+plus variance, sum of squares, and standard deviation.
+
+    extended_stats:field_name
+
+Results in
+
+    {
+        "extended_stats.field_names": {
+            "extended_stats": {
+                "field": "field_name"
+            }
+        }
+    }
+
+=item B<percentiles>
+
+Computes percentiles for the enclosing bucket. The positional parameter is
+interpretted at the percents computed.  If ommitted, the percentiles computed
+will be: 25, 50, 75, 90.
+
+    percentiles:field_name:75,95,99
+
+Results in
+
+    {
+        "percentiles.field_names": {
+            "percentiles": {
+                "field": "field_name",
+                "percents": [ 75, 95, 99 ]
+            }
+        }
+    }
+
+=item B<geo_centroid>
+
+Computes center of a group of geo points. No positional parameters supported.
+
+    geo_centroid:field_name
+
+Results in
+
+    {
+        "geo_centroid.field_names": {
+            "geo_centroid": {
+                "field": "field_name"
+            }
+        }
+    }
+
+=back
+
 =for Pod::Coverage es_flatten_aggs
 
 =head1 AUTHOR
